'use strict';

const {Gio, GObject} = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const QuickSettings = imports.ui.quickSettings;
const QuickSettingsMenu = imports.ui.main.panel.statusArea.quickSettings;

const gettextDomain = Me.metadata['gettext-domain'];
const Gettext = imports.gettext;
const Domain = Gettext.domain(gettextDomain);
const _ = Domain.gettext;
const ngettext = Domain.ngettext;

const Battery = Me.imports.libs.Battery;
const Utils = Me.imports.libs.Utils;

const IconsFolderPath = Me.dir.get_child('icons/42').get_path();

const STATUS_INACTIVE = 0;
const STATUS_ACTIVE = 1;
const STATUS_UNKNOWN = 2;

const ThresholdToggle = GObject.registerClass({
    GTypeName: 'ThresholdToggle',
    Signals: {
        'status-change': {
            param_types: [GObject.TYPE_INT],
        },
    },
}, class ThresholdToggle extends QuickSettings.QuickMenuToggle {
    _init() {
        super._init({
            label: _('Battery Threshold'),
            gicon: Gio.icon_new_for_string(IconsFolderPath + "/threshold-unknown-symbolic.svg"),
            toggleMode: false,
        })

        this.settings = ExtensionUtils.getSettings();

        this._bat0 = new Battery.Threshold(0);
        this._bat1 = new Battery.Threshold(1);

        this.menu.setHeader(
            Gio.icon_new_for_string(IconsFolderPath + "/threshold-app-symbolic.svg"), 
            _('Battery Threshold')/*, 
            'Optional Subtitle'*/
        );

        // Unavailable
        this._bat0Status = new Utils.BatStatus();
        this._bat1Status = new Utils.BatStatus();
        this._unavailableLabel = new PopupMenu.PopupImageMenuItem(_('Thresholds not available'), Gio.icon_new_for_string(IconsFolderPath + "/threshold-unknown-symbolic.svg"));
        this._unavailableLabel.sensitive = false;
        this.menu.addMenuItem(this._unavailableLabel);

        this._bat0Section = new PopupMenu.PopupMenuSection();

        // Main bat toggle menu item
        this._bat0Toggle = new PopupMenu.PopupImageMenuItem(_('Enable thresholds (BAT0)'), Gio.icon_new_for_string(IconsFolderPath + "/threshold-active-symbolic.svg"));
        this._bat0Toggle.connect('activate', () => {
            if (this._bat0.isAvailable) {
                this._bat0.toggleThreshold().then(
                    this._mainSuccess.bind(this), 
                    this._mainFailure.bind(this)
                );
            }
        });
        this._bat0Toggle.add_child(this._bat0Status);
        this._applyBat0 = new PopupMenu.PopupImageMenuItem(_('Apply (BAT0)'), 'view-refresh-symbolic');
        this._applyBat0.connect('activate', () => {
            if (this._bat0.isAvailable) {
                this._bat0.enableThreshold().then(
                    this._mainSuccess.bind(this), 
                    this._mainFailure.bind(this)
                );
            }
        });
        this._bat0Section.addMenuItem(this._bat0Toggle);
        this._bat0Section.addMenuItem(this._applyBat0);
        this.menu.addMenuItem(this._bat0Section);

        this._bat1Section = new PopupMenu.PopupMenuSection();
        // Dock bat toggle menu item
        this._bat1Toggle = new PopupMenu.PopupImageMenuItem(_('Enable thresholds (BAT1)'), Gio.icon_new_for_string(IconsFolderPath + "/threshold-active-symbolic.svg"));
        this._bat1Toggle.connect('activate', () => {
            if (this._bat1.isAvailable) {
                this._bat1.toggleThreshold().then(
                    this._secondarySuccess.bind(this), 
                    this._secondaryFailure.bind(this)
                );
            }
        });
        this._bat1Toggle.add_child(this._bat1Status);
        this._applyBat1 = new PopupMenu.PopupImageMenuItem(_('Apply (BAT1)'), 'view-refresh-symbolic');
        this._applyBat1.connect('activate', () => {
            if (this._bat1.isAvailable) {
                this._bat1.enableThreshold().then(
                    this._secondarySuccess.bind(this), 
                    this._secondaryFailure.bind(this)
                );
            }
        });
        this._bat1Section.addMenuItem(this._bat1Toggle);
        this._bat1Section.addMenuItem(this._applyBat1);
        this.menu.addMenuItem(this._bat1Section);

        // Add an entry-point for more settings
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        const settingsItem = this.menu.addAction(_('Thresholds settings'),
            () => ExtensionUtils.openPrefs());
            
        // Ensure the settings are unavailable when the screen is locked
        settingsItem.visible = Main.sessionMode.allowSettings;
        this.menu._settingsActions[Me.uuid] = settingsItem;

        this.updateGUI();

        this._bat0.enableMonitors(this.updateGUI.bind(this));
        this._bat1.enableMonitors(this.updateGUI.bind(this));

        this.connectObject('clicked', () => this._toggleMode(), this);

        this._settingsHandler = this.settings.connect('changed', this.updateGUI.bind(this));
    }

    _toggleMode() {
        // TODO: Here it will ask for credentials twice if you have the dock connected... Change that
        // To change this, it will be necessary to modify the "driver" so that it can execute 
        // all the commands (main battery and dock) at the same time with a single request 
        // for credentials... This would imply several changes... Patience...
        if (this.checked) {
            if (this._bat0.isAvailable) {
                this._bat0.disableThreshold().then(
                    this._mainSuccess.bind(this), 
                    this._mainFailure.bind(this)
                );
            }
            if (this._bat1.isAvailable) {
                this._bat1.disableThreshold().then(
                    this._secondarySuccess.bind(this), 
                    this._secondaryFailure.bind(this)
                );
            }
        } else {
            if (this._bat0.isAvailable) {
                this._bat0.enableThreshold().then(
                    this._mainSuccess.bind(this), 
                    this._mainFailure.bind(this)
                );
            }
            if (this._bat1.isAvailable) {
                this._bat1.enableThreshold().then(
                    this._secondarySuccess.bind(this), 
                    this._secondaryFailure.bind(this)
                );
            }
        }
    }

    /**
     * Return full path icon.
     * 
     * @param {string} name Name of icon
     * @returns {string} Icon full path
     */
    _getIconPath(name) {
        const iconsTheme = this.settings.get_enum('icons-theme');
        return IconsFolderPath + '/' + name + (iconsTheme == 0 ? '' : '-symbolic') + '.svg';;
    }

    /**
     * Success callback.
     * 
     * @param {[number, number]} result Threshold values applied
     * @param {boolean} isDock True if it is the secondary battery
     */
     _thresholdSuccessCallback(result, isDock) {
        if (!this._showNotifications) return
        let message = '';
        let iconPath = '';
        if (result[0] === 0 && result[1] === 100) {
            iconPath = this._getIconPath('threshold-inactive');
            if (isDock) {
                message = _('Secondary battery thresholds disabled')
            } else {
                message = _('Main battery thresholds disabled')
            }
        } else {
            iconPath = this._getIconPath('threshold-active');
            if (isDock) {
                // TRANSLATORS: The first %d is the start value and the second %d is the stop value. The string %% is the percent symbol (may need to be escaped depending on the language)
                message = _('Secondary battery charge thresholds enabled at %d/%d %%').format(result[0], result[1]);
            } else {
                // TRANSLATORS: The first %d is the start value and the second %d is the stop value. The string %% is the percent symbol (may need to be escaped depending on the language)
                message = _('Main battery charge thresholds enabled at %d/%d %%').format(result[0], result[1]);
            }
        }
        const icon = Gio.icon_new_for_string(iconPath);
        Utils.notify(_('Battery Threshold'), message, icon);
    }

    _mainSuccess(result) {
        this._thresholdSuccessCallback(result, false);
    }

    _secondarySuccess(result) {
        this._thresholdSuccessCallback(result, true);
    }

    /**
     * Failure callback.
     * 
     * @param {Error} error Error applying thresholds
     * @param {boolean} isDock True if it is the secondary battery
     */
    _thresholdFailureCallback(error, isDock) {
        if (!this._showNotifications) return
        if (error !== null) {
            const icon = Gio.icon_new_for_string(this._getIconPath('threshold-error'));
            let message = error.message;
            if (isDock) {
                message = _('Dock battery error: ') + message;
            } else {
                message = _('Main battery error: ') + message;
            }
            Utils.notify(_('Battery Threshold'), message, icon);
        }
        this.updateGUI();
    }

    _mainFailure(error) {
        this._thresholdFailureCallback(error, false);
    }

    _secondaryFailure(error) {
        this._thresholdFailureCallback(error, true);
    }

    /**
     * Update GUI elements.
     */
    updateGUI() {
        //const indicatorMode = this.settings.get_enum('indicator-mode');
        const showValues = this.settings.get_boolean('show-current-values');
        const showTooltips = this.settings.get_boolean('show-tooltips');
        this._showNotifications = this.settings.get_boolean('show-notifications');
        const startBat0 = this.settings.get_int('start-bat0');
        const stopBat0 = this.settings.get_int('stop-bat0');
        const startBat1 = this.settings.get_int('start-bat1');
        const stopBat1 = this.settings.get_int('stop-bat1');

        this._bat0Status.visible = showValues;
        this._bat1Status.visible = showValues;

        if (this._bat0.isAvailable || this._bat1.isAvailable) {
            // Main
            this._bat0Toggle.visible = this._bat0.isAvailable;
            this._bat0Toggle.label.text = this._bat0.isEnabled ? _('Disable thresholds (BAT0)') : _('Enable thresholds (BAT0)');
            const icon0 = this._bat0.isEnabled ? Gio.icon_new_for_string(IconsFolderPath + "/threshold-active-symbolic.svg") : Gio.icon_new_for_string(IconsFolderPath + "/threshold-inactive-symbolic.svg");
            this._bat0Toggle.setIcon(icon0);

            if (this._bat0.isAvailable) {
                if (this._bat0.isEnabled) {
                    this._bat0Status.icon.gicon = Gio.icon_new_for_string(this._getIconPath('threshold-active'));
                    if (startBat0 != this._bat0.currentStartValue || 
                        stopBat0 != this._bat0.currentStopValue) {
                        this._applyBat0.visible = true;
                    } else {
                        this._applyBat0.visible = false;
                    }
                } else {
                    this._bat0Status.icon.gicon = Gio.icon_new_for_string(this._getIconPath('threshold-inactive'));
                    this._applyBat0.visible = false;
                }
                // TRANSLATORS: The first %d is the start value and the second %d is the stop value. The string %% is the percent symbol (may need to be escaped depending on the language)
                this._bat0Status.label.text = _('%d/%d %%').format(this._bat0.currentStartValue, this._bat0.currentStopValue);
                // TRANSLATORS: The first %d is the start value and the second %d is the stop value. The string %% is the percent symbol (may need to be escaped depending on the language)
                this._bat0Status.tooltip.text = showTooltips ? _('Main battery (%d/%d %%)').format(this._bat0.currentStartValue, this._bat0.currentStopValue) : null;                    
            } else {
                this._bat0Status.icon.gicon = Gio.icon_new_for_string(this._getIconPath('threshold-unknown'));
                this._applyBat0.visible = false;
            }

            // Dock
            this._bat1Toggle.visible = this._bat1.isAvailable;
            this._bat1Toggle.label.text = this._bat1.isEnabled ? _('Disable thresholds (BAT1)') : _('Enable thresholds (BAT1)');
            const icon1 = this._bat1.isEnabled ? Gio.icon_new_for_string(IconsFolderPath + "/threshold-active-symbolic.svg") : Gio.icon_new_for_string(IconsFolderPath + "/threshold-inactive-symbolic.svg");
            this._bat1Toggle.setIcon(icon1);

            if (this._bat1.isAvailable) {
                if (this._bat1.isEnabled) {
                    this._bat1Status.icon.gicon = Gio.icon_new_for_string(this._getIconPath('threshold-active'));
                    if (startBat1 != this._bat1.currentStartValue || 
                        stopBat1 != this._bat1.currentStopValue) {
                        this._applyBat1.visible = true;
                    } else {
                        this._applyBat1.visible = false;
                    }
                } else {
                    this._bat1Status.icon.gicon = Gio.icon_new_for_string(this._getIconPath('threshold-inactive'));
                    this._applyBat1.visible = false;
                }
                // TRANSLATORS: The first %d is the start value and the second %d is the stop value. The string %% is the percent symbol (may need to be escaped depending on the language)
                this._bat1Status.label.text = _('%d/%d %%').format(this._bat1.currentStartValue, this._bat1.currentStopValue);
                // TRANSLATORS: The first %d is the start value and the second %d is the stop value. The string %% is the percent symbol (may need to be escaped depending on the language)
                this._bat1Status.tooltip.text = showTooltips ? _('Dock battery (%d/%d %%)').format(this._bat1.currentStartValue, this._bat1.currentStopValue) : null;     
            } else {
                this._applyBat1.visible = false;
                this._bat1Status.icon.gicon = Gio.icon_new_for_string(this._getIconPath('threshold-unknown'));
            }
            
            this._unavailableLabel.visible = false;

            if (this._bat0.isEnabled || this._bat1.isEnabled) {
                this.gicon = Gio.icon_new_for_string(IconsFolderPath + "/threshold-active-symbolic.svg");
                this.checked = true;
                this.emit('status-change', STATUS_ACTIVE);
            } else {
                this.checked = false;
                this.gicon = Gio.icon_new_for_string(IconsFolderPath + "/threshold-inactive-symbolic.svg");
                this.emit('status-change', STATUS_INACTIVE);
            }
        } else {
            this._unavailableLabel.visible = true;
            this._bat0Toggle.visible = false;
            this._bat1Toggle.visible = false;
            this._applyBat0.visible = false;
            this._applyBat1.visible = false;
            this.checked = false;
            this.gicon = Gio.icon_new_for_string(IconsFolderPath + "/threshold-unknown-symbolic.svg");
            this.emit('status-change', STATUS_UNKNOWN);
        }
    }

    destroy() {
        this.settings.discconect(this._settingsHandler);
        this._bat0.destroy();
        this._bat1.destroy();
        this.menu.destroy();
        super.destroy();
    }
});

var ThresholdIndicator = GObject.registerClass({
    GTypeName: 'ThresholdIndicator',
}, class ThresholdIndicator extends QuickSettings.SystemIndicator {
    _init() {
        super._init();
        this._indicator = this._addIndicator();
        this._indicator.gicon = Gio.icon_new_for_string(IconsFolderPath + "/threshold-app-symbolic.svg");

        this._toggle = new ThresholdToggle();

        this.quickSettingsItems.push(this._toggle);

        this.settings = ExtensionUtils.getSettings();

        const handlerId = this._toggle.connect('status-change', (_, newStatus) => {
            const indicatorMode = this.settings.get_enum('indicator-mode');
            let indicatorIconPath = null;
            switch (newStatus) {
                case STATUS_ACTIVE:
                    indicatorIconPath = this._getIconPath('threshold-active');
                    break;
                case STATUS_INACTIVE:
                    indicatorIconPath = this._getIconPath('threshold-inactive');
                    break;
                default:
                    indicatorIconPath = this._getIconPath('threshold-unknown');
                    break;
            }
            this._indicator.gicon = Gio.icon_new_for_string(indicatorIconPath);
            switch (indicatorMode) {
                case 1:
                    this._indicator.visible = (newStatus == STATUS_INACTIVE);
                    break;
                case 2:
                    this._indicator.visible = true;
                    break;
                case 3:
                    this._indicator.visible = false;
                    break;
                default:
                    this._indicator.visible = (newStatus == STATUS_ACTIVE);
                    break;
            }
        })

        this.connect('destroy', () => {
            this.quickSettingsItems.forEach(item => item.destroy());
            this._toggle.discconect(handlerId);
        });

        //QuickSettingsMenu._indicators.add_child(this); // Right
        QuickSettingsMenu._indicators.insert_child_at_index(this, 0); // Left
        QuickSettingsMenu._addItems(this.quickSettingsItems);

        this._toggle.updateGUI();
    }

    /**
     * Return full path icon.
     * 
     * @param {string} name Name of icon
     * @returns {string} Icon full path
     */
    _getIconPath(name) {
        const iconsTheme = this.settings.get_enum('icons-theme');
        return IconsFolderPath + '/' + name + (iconsTheme == 0 ? '' : '-symbolic') + '.svg';;
    }
});